<?php

namespace Drupal\entityreference_extensions;

use Drupal\Core\Field\Plugin\Field\FieldType\EntityReferenceItem;

/**
 * Trait EntityReferenceDeltaFilterTrait
 *
 * provides a filter callback and other helpers that are used by
 * multiple classes that do not and can not inherit from the same parent
 *
 * @package Drupal\entityreference_extensions
 */
trait EntityReferenceDeltaFilterTrait {

  public function delta_filter(EntityReferenceItem $item) {
    $delta = intval($item->getName());
    $offset = intval($this->getSetting('offset'));
    if($this->getSetting('reverse')) {
      if ($delta > ($this->no_items - 1 - $offset)) {
        return FALSE;
      }
      if ($delta >= ($this->no_items - 1 - $offset) - $this->getSetting('delta')) {
        return TRUE;
      }
      else {
        return FALSE;
      }
    }
    else {
      if ($delta > $this->getSetting('delta')+$offset) {
        return FALSE;
      }
      if ($delta < $offset) {
        return FALSE;
      }
      return TRUE;
    }
  }

  /**
   * check whether field cardinality is not 1 (detect multi value field)
   *
   * @return bool
   */
  public function field_cardinality_valid() {
    if ($this->fieldDefinition->getFieldStorageDefinition()->getCardinality() != 1) {
      return TRUE;
    }
    else {
      return FALSE;
    }
  }

  /**
   * provide the setting form element for limiting entity reference items
   *
   * @return array - form element
   */
  public function delta_setting() {
    $options = [
      'all' => $this->t('All'),
    ];
    for ($i = 0; $i < $this->get_cardinality_counter(); $i++) {
      $options[$i] = $i + 1;
    }
    return [
      '#type' => 'select',
      '#title' => $this->t('Limit elements'),
      '#options' => $options,
      '#default_value' => $this->getSetting('delta'),
    ];
  }

  /**
   * provide the setting form element for reverse order of filtered items
   *
   * @return array - form element
   */
  public function reverse_setting() {
    return [
      '#type' => 'checkbox',
      '#title' => $this->t('Reverse order'),
      '#description' => $this->t('Use this if you want the last item, last two items, etc.'),
      '#default_value' => $this->getSetting('reverse')
    ];
  }

  /**
   * provide the setting form element for offsetting of filtered items
   *
   * @return array - form element
   */
  public function offset_setting() {
    $options = [
      '0' => $this->t('None'),
    ];
    for ($i = 1; $i < $this->get_cardinality_counter(); $i++) {
      $options[$i] = $i;
    }
    return [
      '#type' => 'select',
      '#title' => $this->t('Offset'),
      '#options' => $options,
      '#default_value' => $this->getSetting('offset'),
    ];
  }

  public function get_cardinality_counter() {
    if ( $this->fieldDefinition->getFieldStorageDefinition()->getCardinality() == -1) {
      return \Drupal::config('entityreference_extensions.settings')->get('unlimitedcounter') ? \Drupal::config('entityreference_extensions.settings')->get('unlimitedcounter') : 10;
    }
    else {
      return $this->fieldDefinition->getFieldStorageDefinition()->getCardinality();
    }
  }
}
