<?php

namespace Drupal\entityreference_extensions\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldFormatter\EntityReferenceEntityFormatter;
use Drupal\Core\Form\FormStateInterface;
use Drupal\entityreference_extensions\EntityReferenceDeltaFilterTrait;

/**
 * Plugin implementation of the 'entity reference rendered entity delta' formatter.
 *
 * @FieldFormatter(
 *   id = "entity_reference_entity_view_delta",
 *   label = @Translation("Rendered entity (delta)"),
 *   description = @Translation("Display the referenced entities rendered by entity_view(). Limit multi value field by delta."),
 *   field_types = {
 *     "entity_reference"
 *   }
 * )
 */
class EntityReferenceEntityDeltaFormatter extends EntityReferenceEntityFormatter {

  use EntityReferenceDeltaFilterTrait;

  /**
   * @var int - the number of items for current entity
   */
  protected $no_items;

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
        'delta' => 'all',
        'reverse' => FALSE,
        'offset' => 0
      ] + parent::defaultSettings();
  }
  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = parent::settingsForm($form, $form_state);
    if ($this->field_cardinality_valid()) {
      $elements['delta'] = $this->delta_setting();
      $elements['reverse'] = $this->reverse_setting();
      $elements['offset'] = $this->offset_setting();
    }
    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();
    if ($this->field_cardinality_valid()) {
      $summary[] = $this->getSetting('delta') == 'all' ? t('Show all') : t('Show @item', ['@item' => $this->getSetting('delta') + 1]);
      $summary[] = $this->getSetting('reverse') ? t('Reverse order') : '';
      $summary[] = $this->getSetting('offset') == 0 ? t('No offset') : t('Offset @item', ['@item' => $this->getSetting('offset')]);
    }
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    if($this->getSetting('delta') != 'all' && $this->field_cardinality_valid()) {
      $this->no_items = count($items);
      $items->filter([$this, 'delta_filter']);
    }
    $elements = parent::viewElements($items, $langcode);
    return $elements;
  }


}
